/*
 * @description: Example applications for Impact Acquire
 * @copyright: Copyright (C) 2006 - 2013 Balluff GmbH
 * @authors: APIs and drivers development team at Balluff GmbH
 * @initial date: 2006-10-20
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next paragraph) shall be included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

//----------------------------------------------------------------------------------------
#ifndef aviexceptionH
#define aviexceptionH aviexceptionH
//----------------------------------------------------------------------------------------
#include <string>

//----------------------------------------------------------------------------------------
class AVIException
//----------------------------------------------------------------------------------------
{
public:
    virtual const char* what() const = 0;
};

//----------------------------------------------------------------------------------------
class AEUnsupportedCodec : public AVIException
//----------------------------------------------------------------------------------------
{
public:
    const char* what() const
    {
        return "Unsupported codec";
    }
};

//----------------------------------------------------------------------------------------
class AVIWrapperException : public AVIException
//----------------------------------------------------------------------------------------
{
    std::string m_errorString;
public:
    AVIWrapperException( const char* pError ) : m_errorString( pError ) {}
    const char* what() const
    {
        return m_errorString.c_str();
    }
};

#endif // aviexceptionH