/*
 * @description: Example applications for Impact Acquire
 * @copyright: Copyright (C) 2006 - 2019 Balluff GmbH
 * @authors: APIs and drivers development team at Balluff GmbH
 * @initial date: 2006-10-20
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next paragraph) shall be included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

//----------------------------------------------------------------------------------------
#ifndef aviwrapperH
#define aviwrapperH aviwrapperH
//----------------------------------------------------------------------------------------
#include "avihelper.h"

//----------------------------------------------------------------------------------------
class AVIWrapper
//----------------------------------------------------------------------------------------
{
    // member data
    /// Keeps track about the number of images stored in the actual stream
    int                 m_AVIStreamFrameCounter;
    /// pointer to the structure describing the AVI-file
    PAVIFILE            m_pAVIFile;
    /// pointer to an uncompressed AVI stream
    PAVISTREAM          m_pAVIStream;
    /// pointer to a compressed AVI stream (this is used when a compression handler is used)
    PAVISTREAM          m_pAVIStreamCompressed;
    /// A structure containing information about the AVI stream
    AVISTREAMINFO       m_AVIStreamInfo;
    /// A structure for storing the compression options of the AVI stream
    AVICOMPRESSOPTIONS  m_AVICompressionOptions;
    /// The used codec
    DWORD               m_codec;
    /// Usage count for this class
    static unsigned int m_usageCount;
    // functions
    void                CloseStreamsAndFiles( void );
    void                SetupStreamStructs( BITMAPINFOHEADER& BmpHeader, int w, int h, int bitcount, DWORD sampleRate, DWORD quality, const char* pName = "default", CODEC_T codec = codecMax );
    /// Do not allow copy construction
    AVIWrapper( const AVIWrapper& scr );
public:
    // construction/destruction
    explicit AVIWrapper( const char* pFilename = 0, UINT mode = OF_READ );
    ~AVIWrapper( void );

    // user interface
    void CloseAVIFile( void );
    void CreateAVIStreamFromDIBs( int w, int h, int bitcount, DWORD sampleRate, DWORD quality, const char* pName = "default", CODEC_T codec = codecMax );
    void CreateAVIStreamFromJPEGs( int w, int h, int bitcount, DWORD sampleRate, DWORD quality, const char* pName = "default" );
    const char* GetStreamName( void ) const
    {
        return m_AVIStreamInfo.szName;
    }
    bool UsesCompressionHandler( void ) const
    {
        return m_codec != CodecToFourccCode( codecNoCompression );
    }
    void OpenAVIFile( const char* pFilename, UINT mode = OF_READ );
    void SaveDataToAVIStream( unsigned char* pData, int size );
};

#endif // aviwrapperH